<?php
//
// Bulk Modifications Component
// Copyright (c) 2010-2014 Nagios Enterprises, LLC.  All rights reserved.
//

// Include the helper file
require_once(dirname(__FILE__) . '/../componenthelper.inc.php');

$bulkmodifications_component_name = "bulkmodifications";

// Run the initialization function
bulkmodifications_component_init();

////////////////////////////////////////////////////////////////////////
// COMPONENT INIT FUNCTIONS
////////////////////////////////////////////////////////////////////////

function bulkmodifications_component_init()
{
    global $bulkmodifications_component_name;
    $versionok = bulkmodifications_component_checkversion();
    $desc = gettext("This component allows administrators to submit bulk configurations changes for selected hosts and services. ");

    if (!$versionok) {
        $desc = "<b>" . gettext("Error: This component requires Nagios XI 2012R1.0 or later.") . "</b>";
    }

    // All components require a few arguments to be initialized correctly.
    $args = array(
        COMPONENT_NAME => $bulkmodifications_component_name,
        COMPONENT_VERSION => '1.2.2',
        COMPONENT_DATE => '12/26/2014',
        COMPONENT_AUTHOR => "Nagios Enterprises, LLC",
        COMPONENT_DESCRIPTION => $desc,
        COMPONENT_TITLE => "Bulk Modifications",
        COMPONENT_PROTECTED => true,
        COMPONENT_TYPE => COMPONENT_TYPE_CORE
    );

    // Register this component with XI
    register_component($bulkmodifications_component_name, $args);

    // Register the addmenu function
    if ($versionok) {
        register_callback(CALLBACK_MENUS_INITIALIZED, 'bulkmodifications_component_addmenu');
    }
}


///////////////////////////////////////////////////////////////////////////////////////////
// MISC FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////

function bulkmodifications_component_checkversion()
{
    if (!function_exists('get_product_release')) {
        return false;
    }
    
    // Requires greater than 2011R3.1
    if (get_product_release() < 215) {
        return false;
    }

    return true;
}

function bulkmodifications_component_addmenu($arg = null)
{
    global $bulkmodifications_component_name;
    global $menus;

    // Retrieve the URL for this component
    $urlbase = get_component_url_base($bulkmodifications_component_name);

    // Add this to the core config manager menu
    add_menu_item(MENU_CORECONFIGMANAGER, array(
        "type" => "link",
        "title" => gettext("Bulk Modifications"),
        "id" => "menu-coreconfigmanager-bulkmodifications",
        "order" => 802.7,
        "opts" => array(
            "href" => $urlbase . "/bulkmodifications.php",
        )
    ));

    // Add to the new CCM
    add_menu_item(MENU_CCM, array(
        "type" => "link",
        "title" => gettext("Bulk Modifications"),
        "id" => "menu-ccm-bulkmodifications",
        "order" => 802.7,
        "opts" => array(
            "href" => $urlbase . "/bulkmodifications.php",
        )
    ));
}