#!/bin/bash
#
# Manage Services (start/stop/restart)
# =====================
# Built to allow start/stop/restart of services using the proper method based on
# the actual version of operating system.
#
# Examples:
# ./manage_services.sh start httpd
# ./manage_services.sh restart mysqld
# ./manage_services.sh checkconfig nagios
#

# load in xi-sys information learned during install
DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
source $DIR/../var/xi-sys.cfg

# Things you can and can't do
first=("start" "stop" "restart" "status" "reload" "checkconfig")
second=("httpd" "mysqld" "nagios" "ndo2db" "npcd" "snmptt")

# Helper functions
# -----------------------

contains () {
    local array="$1[@]"
    local seeking=$2
    local in=1
    for element in "${!array}"; do
        if [[ $element == $seeking ]]; then
            in=0
            break
        fi
    done
    return $in
}

# Verify to avoid abuse
# -----------------------

# Check to verify the proper usage format
# ($1 = action, $2 = service name)

if ! contains first "$1"; then
    echo "First parameter must be one of: ${first[*]}"
    exit 1
fi

if ! contains second "$2"; then
    echo "Second parameter must be one of: ${second[*]}"
    exit 1
fi

action=$1
service=$2

# Run the command
# -----------------------

# CentOS / Red Hat

if [ "$distro" == "CentOS" ] || [ "$distro" == "RedHatEnterpriseServer" ]; then
    if [ "$dist" == "el7" ]; then
        if [ "$service" == "mysqld" ]; then
            service=$mysqld
        fi
        `which systemctl` $action $service
    elif [ ! `command -v service` ]; then
        /etc/init.d/$service $action
    else
        `which service` $service $action
    fi
fi

# OpenSUSE / SUSE Enterprise

# Ubuntu / Debian

# Others?